//====================================================
//
//  URL component by ztn
//
//  ztn@planetquake.com
//  http://www.planetquake.com/ztn
//
//----------------------------------------------------
//  Version history:
//    9/6/2000 - 1st public release
//====================================================

unit urls;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ShellAPI;

const
  crExplorerCursor = -20123;

type
  TUrlFailedEvent = procedure (Sender: TObject; URL: string; ErrorCode: Integer) of object;

  TUrl = class(TCustomStaticText)
  private
    FActiveLinkColor: TColor;
    FActiveLinkStyle: TFontStyles;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnUrlFailed: TUrlFailedEvent;
    FURL: string;
    SaveColor: TColor;
    SaveStyle: TFontStyles;
    procedure CMMouseEnter(var Msg:TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg:TMessage); message CM_MOUSELEAVE;
    procedure SetURL(const AURL: string);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
    function Execute: Boolean; virtual;
  published
    property ActiveLinkColor: TColor read FActiveLinkColor write FActiveLinkColor;
    property ActiveLinkStyle: TFontStyles read FActiveLinkStyle write FActiveLinkStyle;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter
      write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave
      write FOnMouseLeave;
    property OnUrlFailed: TUrlFailedEvent read FOnUrlFailed write FOnUrlFailed;
    property URL: string read FURL write SetURL;
    
    // From TCustomStaticText
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

{ Global }

procedure Register;
begin
  RegisterComponents('Custom', [TUrl]);
end;

{ TURL }

constructor TUrl.Create;
begin
  inherited Create(AOwner);
  FURL := '';
  ShowHint := True;
  FActiveLinkColor := clBlue;
  FActiveLinkStyle := [fsUnderline];
  Font.Color := clNavy;
  if (AOwner <> nil) and (AOwner is TCustomForm) then
    Font.Name := (AOwner as TCustomForm).Font.Name;
  Screen.Cursors[crExplorerCursor] := LoadCursor(0, IDC_HAND);
  Cursor := crExplorerCursor;
  SaveColor := Font.Color;
  SaveStyle := Font.Style;
end;

procedure TUrl.Click;
begin
  inherited;
  if not Assigned(OnClick) then
    Execute;
end;

function TUrl.Execute: Boolean;
var
  ResVal: Integer;
begin
  Result := False;
  if FURL <> '' then
  begin
    ResVal := ShellExecute(GetDesktopWindow, 'open', PChar(FURL),
      nil, nil, SW_SHOWNORMAL);
    Result := ResVal > 32;
    if not Result and Assigned(FOnUrlFailed) then
      FOnUrlFailed(Self, FURL, ResVal);
  end;
end;

procedure TUrl.CMMouseEnter(var Msg:TMessage);
begin
  SaveColor := Font.Color;
  SaveStyle := Font.Style;
  Font.Color := FActiveLinkColor;
  Font.Style := FActiveLinkStyle;
  inherited;
  if Assigned(FOnMouseEnter) then
    FOnMouseEnter(Self);
end;

procedure TUrl.CMMouseLeave(var Msg:TMessage);
begin
  Font.Color := SaveColor;
  Font.Style := SaveStyle;
  inherited;
  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(Self);
end;

procedure TUrl.SetURL(const AURL: string);
begin
  if AURL <> FURL then
  begin
    if ((Hint = FURL) or (Hint = '')) and ShowHint then Hint := AURL;
    if (Caption = FURL) or (Caption = '') then Caption := AURL;
    FURL := AURL;
    Update;
  end;
end;

end.
